/*
 * LED_Dimmer.c
 *
 * Created: 02.04.2017 10:32:33
 *  Author: Frank
 * 
 * Prozessor ATTiny45 oder 85(fr mehr als zwei Protokolle)
 * F_CPU 8MHz
 *
 */ 

/*
	EEPROM-Belegung
	
	0x00	Prfbyte1
	0x01	Prfbyte2
	0x02	Starthelligkeit
	
	IR-Codes
	0x03	Protokoll
	0x04/05	Adresse
	Datenbytes
	0x06	Ein/Aus
	0x07	1
	0x08	2
	0x09	3
	0x0a	4
	0x0b	5
	0x0c	6
	0x0d	7
	0x0e	8
	0x0f	9
	0x10	0
	0x11	Heller
	0x12	Dunkler
*/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdlib.h>
#include "irmp.h"


//Taster zum Anlernen der Fernbedienung
#define TASTER	PB2
#define TASTER_GEDRUECKT()	!(PINB & (1<<TASTER))

// Timer fr Zeitbasis
uint32_t tcounter=0;
unsigned savetimer=0;
unsigned timeout=0;
char pwm;				//Helligkeitswert
char protokoll;			//IR-Protokoll
uint16_t adresse;		//IR-Adresse
uint8_t ein;			//Code fr EIN/AUS
uint8_t eins;			//Taste 1
uint8_t zwei;			//Taste 2
uint8_t drei;			//Taste 3
uint8_t vier;			//Taste 4
uint8_t fuenf;			//Taste 5
uint8_t sechs;			//Taste 6
uint8_t sieben;			//Taste 7
uint8_t acht;			//Taste 8
uint8_t neun;			//Taste 9
uint8_t null;			//Taste 0
uint8_t heller;			//Taste Heller
uint8_t dunkler;		//Taste Dunkler

//*********************************************************************************************************
//function for disabling timer
void func_pwm_off()
{
	TCCR0A = 0x00;
	TCCR0B = 0x00;
}
//*********************************************************************************************************
//function used to configure timer 0 and set pwm value 
//timer is configured as 8 bit Phase Correct PWM with a prescalar of 8
void func_pwm_on(char value)
{
	if(value == 0)
	{
		func_pwm_off();
	}
	else
	{
   	 	TCCR0A |= (1 << COM0B1) | (1 << WGM00);
		TCCR0B |= (1 << CS01);
		OCR0B = value ;
	}
}
/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * timer 1 compare handler, called every 1/10000 sec
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
// Timer 1 output compare A interrupt service routine
ISR(TIMER1_COMPA_vect)
{
	irmp_ISR();                                                           // call irmp ISR
	// call other timer interrupt routines...
	tcounter++;
	if (tcounter==10000)
	{
		//diese Routine wird jede Sekunde aufgerufen
		tcounter=0;
		if (savetimer>0x00) savetimer --;
		if (savetimer==0x01)
		{
			//Helligkeitswert abspeichern
			eeprom_write_byte(((uint8_t*)0x02),pwm);
		}
		if (timeout>0x00) timeout --;	//Timeout fr Programmierung
	}
}

void
timer_init (void)
{
	// init Timer1
	OCR1A   =  (F_CPU / F_INTERRUPTS/4) - 1;				// compare value: 1/10000 of CPU frequency
	TCCR1  = (1 << CTC1) | (1 << CS10) | (1 << CS11);		// switch CTC Mode on, set prescaler to 4

	TIMSK  = 1 << OCIE1A;									// OCIE1A: Interrupt by timer compare
}

/***************************************************************
 Funktion Blinken
 
 Lsst die LED die angegebene Anzahl blinken
***************************************************************/
void blink(uint8_t zahl){
uint8_t i;

	for (i=0;i<zahl;i++)
	{
		func_pwm_off();
		_delay_ms(250);
		func_pwm_on(255);	//Lampe blinkt 1x
		_delay_ms(250);
		func_pwm_off();
		_delay_ms(250);
	}
} 

/***************************************************************
 Funktion programm
 
 zum Anlernen des Fernbedienungscodes
 Nacheinander mssen die Tasten Ein/Aus, 1, 2...9, 0, up, down
 gedrckt werden. Nach 10 sec. erfolgt ein Timeout
***************************************************************/
void programm_ir (void) {
IRMP_DATA irmp_data;
uint8_t progdata[16];
uint8_t i;

	blink(2);
	while (TASTER_GEDRUECKT()){}	//Warten, auf loslassen des Tasters
	//Warten auf Fernbedienung Taste Ein/Aus
	timeout=10;		//10 sec bis Timeout
	while (!irmp_get_data((&irmp_data))){
		if (timeout==0) return;
	}
	//Aus dem ersten Telegramm Protokoll, Adresse und den Tastencode abspeichern
	progdata[0] = irmp_data.protocol;
	progdata[1] = irmp_data.address;
	progdata[2] = (irmp_data.address & 0xff00) >> 8;
	progdata[3] = irmp_data.command;
	blink(1);
		
	//nun die restlichen Tasten abfragen
	for (i=4;i<16;i++) {
		timeout=10;		//10 sec bis Timeout
		while (1){
			if (irmp_get_data(&irmp_data)) {
				if (irmp_data.command!=progdata[i-1]) break;	//falls noch alte Taste gedrckt, Schleife fortsetzen, sonst weiter
			}
			if (timeout==0) return;
		}
		//Tastencode abspeichern
		progdata[i] = irmp_data.command;
		blink(1);
	}
	
	
	//Daten in den Arbeitsspeicher bernehmen
	protokoll = progdata[0];
	adresse = progdata[2];
	adresse <<= 8;
	adresse += progdata[1];
	ein=progdata[3];
	eins=progdata[4];
	zwei=progdata[5];
	drei=progdata[6];
	vier=progdata[7];
	fuenf=progdata[8];
	sechs=progdata[9];
	sieben=progdata[10];
	acht=progdata[11];
	neun=progdata[12];
	null=progdata[13];
	heller=progdata[14];
	dunkler=progdata[15];

	//Daten ins eeprom speichern
	for (i=0;i<16;i++) {
		eeprom_write_byte(((uint8_t*)(i+3)),progdata[i]);
	}
}
			


/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * Hauptprogramm
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

int main(void)
{
	IRMP_DATA irmp_data;
	unsigned a,b,c;
	
	DDRB = 0b00000010;			//Setting PWM pin as output, all others as input
	PORTB |= (1<<TASTER);		//set Pullup fr Taster

	irmp_init();                                                              // initialize ir-receiver
	timer_init();                                                             // initialize timer

	sei();

	//Testen, ob EEPROM gltige Werte enthlt, wenn nicht, Defaultwerte eintragen
	a=eeprom_read_byte((uint8_t*)0x00);
	b=eeprom_read_byte((uint8_t*)0x01);
	c=a^b;
	if (!(c==0xff))
	{
		eeprom_write_byte(((uint8_t*)0x00),0x55);		//Prfbyte
		eeprom_write_byte(((uint8_t*)0x01),0xaa);		//Prfbyte
		eeprom_write_byte(((uint8_t*)0x02),0xff);		//Volle Helligkeit als Ausgangswert
		eeprom_write_byte(((uint8_t*)0x03),IRMP_NEC_PROTOCOL);	//schon mal Daten einer Fernbedienung eintragen
		eeprom_write_byte(((uint8_t*)0x04),0x00);
		eeprom_write_byte(((uint8_t*)0x05),0xff);
		eeprom_write_byte(((uint8_t*)0x06),0xc5);
		eeprom_write_byte(((uint8_t*)0x07),0x81);
		eeprom_write_byte(((uint8_t*)0x08),0x83);
		eeprom_write_byte(((uint8_t*)0x09),0xc1);
		eeprom_write_byte(((uint8_t*)0x0a),0x82);
		eeprom_write_byte(((uint8_t*)0x0b),0x80);
		eeprom_write_byte(((uint8_t*)0x0c),0xc0);
		eeprom_write_byte(((uint8_t*)0x0d),0x8d);
		eeprom_write_byte(((uint8_t*)0x0e),0x8f);
		eeprom_write_byte(((uint8_t*)0x0f),0xcd);
		eeprom_write_byte(((uint8_t*)0x10),0x8c);
		eeprom_write_byte(((uint8_t*)0x11),0xd1);
		eeprom_write_byte(((uint8_t*)0x12),0xd0);
	}
	pwm=eeprom_read_byte((uint8_t*)0x02);				//gespeicherten Helligkeitswert als Startwert
	protokoll=eeprom_read_byte((uint8_t*)0x03);
	adresse=eeprom_read_word((uint16_t*)0x0004);
	ein=eeprom_read_byte((uint8_t*)0x06);
	eins=eeprom_read_byte((uint8_t*)0x07);
	zwei=eeprom_read_byte((uint8_t*)0x08);
	drei=eeprom_read_byte((uint8_t*)0x09);
	vier=eeprom_read_byte((uint8_t*)0x0a);
	fuenf=eeprom_read_byte((uint8_t*)0x0b);
	sechs=eeprom_read_byte((uint8_t*)0x0c);
	sieben=eeprom_read_byte((uint8_t*)0x0d);
	acht=eeprom_read_byte((uint8_t*)0x0e);
	neun=eeprom_read_byte((uint8_t*)0x0f);
	null=eeprom_read_byte((uint8_t*)0x10);
	heller=eeprom_read_byte((uint8_t*)0x11);
	dunkler=eeprom_read_byte((uint8_t*)0x12);
	
	while(1)
	{
		if (TASTER_GEDRUECKT()) {
			programm_ir();
			blink(3);
		}


		if (irmp_get_data(&irmp_data))		//einlesen eines IR-Telegramms
		{
			if ((irmp_data.protocol==protokoll) && (irmp_data.address==adresse) && (irmp_data.flags==0))
			{
				//Tasten 0-9 auswerten
				if (irmp_data.command==null) {
					pwm=0;
					savetimer = 0x0a;					
				}
				else if (irmp_data.command==eins)
				{
					pwm=0x1c;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==zwei)
				{
					pwm=0x38;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==drei)
				{
					pwm=0x54;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==vier)
				{
					pwm=0x70;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==fuenf)
				{
					pwm=0x8c;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==sechs)
				{
					pwm=0xa9;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==sieben)
				{
					pwm=0xc6;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==acht)
				{
					pwm=0xe3;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==neun)
				{
					pwm=0xff;
					savetimer = 0x0a;
				}
				else if (irmp_data.command==ein)
				{
					if (pwm>0)
					{
						//LED leutet, Wert bei Bedarf abspeichern und ausschalten
						if (savetimer>0x01)
						{
							eeprom_write_byte(((uint8_t*)0x02),pwm);		//Helligkeit speichern als Ausgangswert
							savetimer=0;
						}
						pwm=0;
					}
					else {
						pwm=eeprom_read_byte((uint8_t*)0x02);	//gespeicherten Helligkeitswert als Startwert
					}
				}
			}
			if ((irmp_data.protocol==protokoll) && (irmp_data.address==adresse) && (irmp_data.command==heller))
			{
				//heller
				if (pwm<0xfa) {
					pwm +=5;
					savetimer = 0x0a;
				}
			}
			if ((irmp_data.protocol==protokoll) && (irmp_data.address==adresse) && (irmp_data.command==dunkler))
			{
				//dunkler
				if (pwm>0x05) {
					pwm -=5;
					savetimer = 0x0a;
				}
			}

		}
		func_pwm_on(pwm);
	}
}